package org.stuartgunter.rep.interceptor;

import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import org.stuartgunter.rep.formatters.MetaTag;
import org.stuartgunter.rep.formatters.RobotInfo;
import org.stuartgunter.rep.formatters.RobotsHeaderFormatter;
import org.stuartgunter.rep.formatters.RobotsMetaFormatter;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * This interceptor looks for {@link org.stuartgunter.rep.annotations.Robot} and
 * {@link org.stuartgunter.rep.annotations.Robots} annotations and ensures that the appropriate
 * response headers and model data are available to conform to the Robot Exclusion Protocol.
 * <p/>
 * The RepInterceptor may be configured to operate in one of three modes:
 * <ul>
 * <li>{@link RepMode#HEADER}: Writes {@code X-Robots-Tag} headers to the response</li>
 * <li>{@link RepMode#META}: Provides a list of {@link MetaTag} objects in the model under the name
 * {@code robotsMetaTags}, which can be used to add HTML meta tags</li>
 * <li>{@link RepMode#BOTH}: Does both of the above. This is the default mode.</li>
 * </ul>
 * <p/>
 * A {@link RobotCombinatorStrategy} is also used to handle cascading definitions of
 * {@link org.stuartgunter.rep.annotations.Robot} and {@link org.stuartgunter.rep.annotations.Robots} annotations.
 */
public class RepInterceptor extends HandlerInterceptorAdapter {

    public enum RepMode { META, HEADER, BOTH }

    private static final Logger LOG = LoggerFactory.getLogger(RepInterceptor.class);

    public static final String ROBOTS_REQUEST_ATTRIBUTE = RepInterceptor.class.getName() + "-ROBOTS";
    public static final String ROBOTS_META_TAGS_ATTRIBUTE_NAME = "robotsMetaTags";
    public static final String ROBOTS_HEADER_NAME = "X-Robots-Tag";

    private RepMode mode = RepMode.BOTH;
    private RobotCombinatorStrategy combinatorStrategy;

    @Resource
    private RobotsHeaderFormatter headerFormatter;

    @Resource
    private RobotsMetaFormatter metaFormatter;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {

        if (!(handler instanceof HandlerMethod)) {
            LOG.error("Object target is not a HandlerMethod. Check that the correct controller was invoked.");
            throw new IllegalArgumentException("Expected type HandlerMethod but found " + handler.getClass());
        }

        final HandlerMethod handlerMethod = (HandlerMethod) handler;

        final List<RobotInfo> combinedRobots = combinatorStrategy.combine(handlerMethod.getMethod());
        if (combinedRobots.isEmpty()) {
            LOG.debug("Skipping HandlerMethod [{}] - no @Robots defined", handlerMethod);
            return true;
        }

        request.setAttribute(ROBOTS_REQUEST_ATTRIBUTE, combinedRobots);

        if (mode == RepMode.BOTH || mode == RepMode.HEADER) {
            writeHeaders(request, response);
        }

        return true;
    }

    private void writeHeaders(HttpServletRequest request, HttpServletResponse response) {
        final List<RobotInfo> robots = (List<RobotInfo>) request.getAttribute(ROBOTS_REQUEST_ATTRIBUTE);

        for (RobotInfo robot : robots) {
            final String headerValue = headerFormatter.format(robot);
            if (headerValue != null) {
                response.addHeader(ROBOTS_HEADER_NAME, headerValue);
            }
        }
    }

    @Override
    public void postHandle(HttpServletRequest request,
                           HttpServletResponse response,
                           Object handler,
                           ModelAndView modelAndView) throws Exception {

        if (modelAndView != null && (mode == RepMode.BOTH || mode == RepMode.META)) {
            writeModel(request, modelAndView);
        }
    }

    private void writeModel(HttpServletRequest request, ModelAndView modelAndView) {
        final List<RobotInfo> robots = (List<RobotInfo>) request.getAttribute(ROBOTS_REQUEST_ATTRIBUTE);
        List<MetaTag> metaTags = Lists.newArrayList();

        for (RobotInfo robot : robots) {
            metaTags.add(metaFormatter.format(robot));
        }

        modelAndView.addObject(ROBOTS_META_TAGS_ATTRIBUTE_NAME, metaTags);
    }

    public void setMode(RepMode mode) {
        this.mode = mode;
    }

    public void setCombinatorStrategy(RobotCombinatorStrategy combinatorStrategy) {
        this.combinatorStrategy = combinatorStrategy;
    }
}
